@@ -11,12 +11,18 @@ import (
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 
+	v1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/util/retry"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
+	"github.com/k8snetworkplumbingwg/kubemacpool/pkg/names"
 	pool_manager "github.com/k8snetworkplumbingwg/kubemacpool/pkg/pool-manager"
+	"github.com/k8snetworkplumbingwg/kubemacpool/tests/kubectl"
+
 	kubevirtv1 "kubevirt.io/client-go/api/v1"
 )
 
@@ -94,6 +100,33 @@ var _ = Describe("[rfe_id:3503][crit:medium][vendor:cnv-qe@redhat.com][level:com
 				}
 			})
 
+			Context("and applying a dryRun vm create request", func() {
+				var macAddress string
+				BeforeEach(func() {
+					By("Creating a vm with dry run mode")
+					vm := CreateVmObject(TestNamespace, false, []kubevirtv1.Interface{newInterface("br", "")}, []kubevirtv1.Network{newNetwork("br")})
+					createOptions := &client.CreateOptions{}
+					client.DryRunAll.ApplyToCreate(createOptions)
+
+					err := testClient.VirtClient.Create(context.TODO(), vm, createOptions)
+					Expect(err).ToNot(HaveOccurred())
+					Expect(net.ParseMAC(vm.Spec.Template.Spec.Domain.Devices.Interfaces[0].MacAddress)).ToNot(BeEmpty(), "Should successfully parse mac address")
+					macAddress = vm.Spec.Template.Spec.Domain.Devices.Interfaces[0].MacAddress
+				})
+
+				It("should not allocate the Mac assigned in the dryRun request into the pool", func() {
+					By("creating a vm with the same mac to make sure the dryRun assigned mac is not occupied on the macpool")
+					vmDryRunOverlap := CreateVmObject(TestNamespace, false, []kubevirtv1.Interface{newInterface("brDryRunOverlap", macAddress)}, []kubevirtv1.Network{newNetwork("brDryRunOverlap")})
+					err := testClient.VirtClient.Create(context.TODO(), vmDryRunOverlap)
+					Expect(err).ToNot(HaveOccurred())
+
+					actualMac, err := net.ParseMAC(vmDryRunOverlap.Spec.Template.Spec.Domain.Devices.Interfaces[0].MacAddress)
+					Expect(err).ToNot(HaveOccurred(), "Should succeed parsing vmDryRunOverlap mac")
+
+					Expect(actualMac.String()).To(Equal(macAddress), "Should successfully parse mac address")
+				})
+			})
+
 			Context("and the client tries to assign the same MAC address for two different vm. Within Range and out of range", func() {
 				Context("When the MAC address is within range", func() {
 					It("[test_id:2166]should reject a vm creation with an already allocated MAC address", func() {
@@ -270,6 +303,7 @@ var _ = Describe("[rfe_id:3503][crit:medium][vendor:cnv-qe@redhat.com][level:com
 					})
 				})
 			})
+
 			Context("and when restarting kubeMacPool and trying to create a VM with the same manually configured MAC as an older VM", func() {
 				It("[test_id:2179]should return an error because the MAC address is taken by the older VM", func() {
 					vm1 := CreateVmObject(TestNamespace, false, []kubevirtv1.Interface{newInterface("br1", "02:00:ff:ff:ff:ff")}, []kubevirtv1.Network{newNetwork("br1")})
@@ -879,6 +913,66 @@ var _ = Describe("[rfe_id:3503][crit:medium][vendor:cnv-qe@redhat.com][level:com
 					})
 				})
 			})
+
+			Context("and creating two VMs with same mac, one on unmanaged (opted out) namespace", func() {
+				var conflictingVM *kubevirtv1.VirtualMachine
+				BeforeEach(func() {
+					macAddress := "02:00:ff:ff:ff:ff"
+
+					By(fmt.Sprintf("Adding a vm with a Mac Address %s in the managed namespace", macAddress))
+					vm1 := CreateVmObject(TestNamespace, false, []kubevirtv1.Interface{newInterface("br1", macAddress)}, []kubevirtv1.Network{newNetwork("br1")})
+					err := testClient.VirtClient.Create(context.TODO(), vm1)
+					Expect(err).ToNot(HaveOccurred())
+					Expect(net.ParseMAC(vm1.Spec.Template.Spec.Domain.Devices.Interfaces[0].MacAddress)).ToNot(BeEmpty(), "Should successfully parse mac address")
+
+					err = addLabelsToNamespace(OtherTestNamespace, map[string]string{vmNamespaceOptInLabel: "ignore"})
+					Expect(err).ToNot(HaveOccurred(), "should be able to add the namespace labels")
+
+					By(fmt.Sprintf("Adding a vm with the same Mac Address %s on the unmanaged namespace", macAddress))
+					conflictingVM = CreateVmObject(OtherTestNamespace, false, []kubevirtv1.Interface{newInterface("br1", macAddress)}, []kubevirtv1.Network{newNetwork("br1")})
+					err = testClient.VirtClient.Create(context.TODO(), conflictingVM)
+					Expect(err).ToNot(HaveOccurred())
+					Expect(net.ParseMAC(conflictingVM.Spec.Template.Spec.Domain.Devices.Interfaces[0].MacAddress)).ToNot(BeEmpty(), "Should successfully parse mac address")
+				})
+
+				It("should detect duplicate macs gauge after restarting kubemacpool, and then mitigating that", func() {
+					By("moving the unmanaged namespace to be managed")
+					err := cleanNamespaceLabels(OtherTestNamespace)
+					Expect(err).ToNot(HaveOccurred(), "should be able to remove the namespace labels")
+
+					By("restarting Kubemacpool to check if there are duplicats macs in the managed namespaces")
+					err = initKubemacpoolParams()
+					Expect(err).ToNot(HaveOccurred())
+
+					metrics, _, err := getMetrics()
+					Expect(err).ToNot(HaveOccurred())
+
+					expectedMetric := "kubevirt_kmp_duplicate_macs"
+					expectedValue := "1"
+					metric := findMetric(metrics, expectedMetric)
+					Expect(metric).ToNot(BeEmpty(), fmt.Sprintf("metric %s does not appear in endpoint scrape", expectedMetric))
+					Expect(metric).To(Equal(fmt.Sprintf("%s %s", expectedMetric, expectedValue)),
+						fmt.Sprintf("metric %s does not have the expected value %s", expectedMetric, expectedValue))
+
+					By("mitigating the conflict by deleting the vm with the conflicting Mac Address")
+					err = testClient.VirtClient.Delete(context.TODO(), conflictingVM)
+					Expect(err).ToNot(HaveOccurred())
+
+					By("restarting Kubemacpool")
+					err = initKubemacpoolParams()
+					Expect(err).ToNot(HaveOccurred())
+
+					By("verifying the conflict has been resolved")
+					metrics, _, err = getMetrics()
+					Expect(err).ToNot(HaveOccurred())
+
+					expectedValue = "0"
+					metric = findMetric(metrics, expectedMetric)
+					Expect(metric).ToNot(BeEmpty(), fmt.Sprintf("metric %s does not appear in endpoint scrape", expectedMetric))
+					Expect(metric).To(Equal(fmt.Sprintf("%s %s", expectedMetric, expectedValue)),
+						fmt.Sprintf("metric %s does not have the expected value %s", expectedMetric, expectedValue))
+				})
+			})
 		})
 	})
 })
@@ -941,3 +1035,40 @@ func deleteVMI(vm *kubevirtv1.VirtualMachine) {
 		return false
 	}, timeout, pollingInterval).Should(BeTrue(), "should eventually fail getting vm with IsNotFound after vm deletion")
 }
+
+func getMetrics() (string, string, error) {
+	podList, err := getManagerPods()
+	if err != nil {
+		return "", "", err
+	}
+
+	stdout, stderr, err := kubectl.Kubectl("exec", "-n", managerNamespace, podList.Items[0].Name, "--",
+		"curl", "-L", "-k", "-s", "http://127.0.0.1:8080/metrics")
+
+	return stdout, stderr, err
+}
+
+func getManagerPods() (*v1.PodList, error) {
+	deployment, err := testClient.KubeClient.AppsV1().Deployments(managerNamespace).Get(context.TODO(), names.MANAGER_DEPLOYMENT, metav1.GetOptions{})
+	if err != nil {
+		return nil, err
+	}
+
+	labelSelector := labels.Set(deployment.Spec.Selector.MatchLabels).String()
+	podList, err := testClient.KubeClient.CoreV1().Pods(managerNamespace).List(context.TODO(), metav1.ListOptions{LabelSelector: labelSelector})
+	if err != nil {
+		return nil, err
+	}
+
+	return podList, err
+}
+
+func findMetric(metrics string, expectedMetric string) string {
+	for _, line := range strings.Split(metrics, "\n") {
+		if strings.HasPrefix(line, expectedMetric+" ") {
+			return line
+		}
+	}
+
+	return ""
+}
